{ Unit : ValEdit.pas
  Diese Unit enthlt ein von TEdit abgeleitetes Dialogelement
  welches verschiedene Eingabefilter anbietet.
  Bentigt die Unit Validate.Pas.

  Filtertypen:

  etString  : Normales Eingabefeld
  etLongint : Ganzzahlige Werte
  etDoule   : Festkommazahlen
  etHex     : Hexadezimal
  etBin     : Binaer

  Hinweis: Dieses Eingabefeld lt leider noch eine Leereingabe zu.
           Solche Fehler mssen dann der Methode zum Schlieen
           des Dialogs abgefangen werden.

  Kommentare und Verbesserungen bitte an:
  Torsten Portoff [100547,2703]

  Freeware.

}

unit Valedit;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls,Validate;

TYPE TValEditTyp = (etString,etLongint,etDouble,etHex,etBin);

type
  TValidateEdit = class(TEdit)
  private
   PValidator  : TValidator;
   FValEditTyp : TValEditTyp; { Validatortyp     }
   FNK         : INTEGER;     { Nachkommastellen }
   PROCEDURE SetValEditTyp(Value : TValEditTyp);
   PROCEDURE SetNK(Value : INTEGER);
   PROCEDURE CNCommand(VAR Message : TWMCommand); MESSAGE CN_Command;
  protected
  public
   Constructor Create(AOwner : TComponent); override;
   Destructor  Destroy; VIRTUAL;
   PROCEDURE Error;
   FUNCTION IsValid : BOOLEAN; VIRTUAL;
   FUNCTION GetDouble  : Double;  VIRTUAL;
   FUNCTION GetLongInt : LongInt; VIRTUAL;
   PROCEDURE SetDouble(Value : Double); VIRTUAL;
   PROCEDURE SetLongInt(Value : LongInt); VIRTUAL;
   PROCEDURE DoExit; override;
  published
   property ValEditTyp : TValEditTyp read FValEditTyp write SetValEditTyp default etString;
   property Dezimalstellen : INTEGER read FNK write SetNK default 2;
  end;

procedure Register;

implementation


FUNCTION KillSpacesInDouble(Value : DOUBLE; NK : INTEGER) : STRING;
VAR S : STRING;
    FeldBreite : INTEGER;
BEGIN
 FeldBreite := trunc(ln(abs(Value))/ln(10.0))+2+NK;
 Str(Value:FeldBreite:NK,S);
 KillSpacesInDouble := S;
END;

FUNCTION TValidateEdit.GetDouble : Double;
VAR Code : INTEGER;
    Value : Double;
BEGIN
 IF (FValEditTyp = etLongInt) OR (FValEditTyp = etDouble) THEN
 BEGIN
  VAL(Text,Value,Code);
  IF Code = 0 THEN
   GetDouble := Value
  ELSE GetDouble := 0;
 END ELSE GetDouble := 0;
END;

FUNCTION TValidateEdit.GetLongInt : LongInt;
VAR Code : INTEGER;
    Value : LongInt;
BEGIN
 IF FValEditTyp = etLongInt THEN
 BEGIN
  VAL(Text,Value,Code);
  IF Code = 0 THEN
   GetLongInt := Value
  ELSE GetLongInt := 0;
 END ELSE GetLongInt := 0;
END;

PROCEDURE TValidateEdit.SetDouble(Value : Double);
BEGIN
 Text := KillSpacesInDouble(Value, FNK);
END;

PROCEDURE TValidateEdit.SetLongInt(Value : LongInt);
VAR HlpStr : STRING;
BEGIN
 STR(Value,HlpStr);
 Text := HlpStr;
END;

Constructor TValidateEdit.Create(AOwner : TComponent);
BEGIN
 inherited Create(AOwner);
 PValidator  := NIL;
 FValEditTyp := etString;
 FNK := 2;
END;

DESTRUCTOR TValidateEdit.Destroy;
BEGIN
 IF PValidator <> NIL THEN PValidator.Free;
 inherited Destroy;
END;

PROCEDURE TValidateEdit.SetNK(Value : INTEGER);
BEGIN
 IF (FNK <> Value) AND (Value > 0) THEN
 BEGIN
  FNK := Value;
 END;
END;

PROCEDURE TValidateEdit.SetValEditTyp(Value : TValEditTyp);
BEGIN
 IF FValEditTyp <> Value THEN
 BEGIN
  IF PValidator <> NIL THEN PValidator.Free;
  FValEditTyp := Value;
  CASE FValEditTyp OF
   etLongInt : PValidator := TFilterValidator.Create(['0'..'9','+','-']);
   etDouble  : PValidator := TFilterValidator.Create(['0'..'9','+','-','.']);
   etHex     : PValidator := TFilterValidator.Create(['0'..'9','A'..'F','a'..'f','+','-']);
   etBin     : PValidator := TFilterValidator.Create(['0','1','+','-']);
  END;
 END;
END;

PROCEDURE TValidateEdit.Error;
BEGIN
 IF NOT IsValid THEN
  PValidator.Error;
END;

FUNCTION TValidateEdit.IsValid : BOOLEAN;
BEGIN
 IF PValidator <> NIL THEN
  IsValid := PValidator.IsValid(Text)
 ELSE IsValid := TRUE;
END;


{ Ruft Error auf, wenn der Focus gewechselt wird, leider auch, wenn der
  Cancelbutoen gedrckt wird }
procedure TValidateEdit.DoExit;
begin
 inherited DoExit;
 IF NOT(IsValid) THEN
 BEGIN
  Error;
  SetFocus;
  exit;
 END;
END;

{ Unterdrckt Fehleingaben, auch durch Einfgen aus der Zwischenablage }
PROCEDURE TValidateEdit.CNCommand(VAR Message : TWMCommand);
VAR OldText : String;
    Error   : BOOLEAN;
    OldSel  : INTEGER;
BEGIN
 inherited;
 OldSel := SelStart;
 if (Message.NotifyCode = EN_CHANGE) THEN
 BEGIN
  IF PValidator <> NIL THEN
  BEGIN
   Oldtext:=Text;
   Error := PValidator.IsValidInput(OldText);
   Text:=OldText;
   IF (OldSel <= Length(Text)) THEN Selstart := Oldsel
   ELSE SelStart := Length(Text);
  END;
 END;
END;

{ ************************************************* }
procedure Register;
begin
 RegisterComponents('Own', [TValidateEdit]);
end;

end.
